@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2005 Jason Allen
- * Copyright (C) 2007 Christian Persch
+ * Copyright © 2005 Jason Allen
+ * Copyright © 2007 Christian Persch
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -14,59 +14,38 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+ * 59 Temple Place, Suite 330, Boston, MA 02110-1301  USA
  */
 
-#include "config.h"
+#include <config.h>
 
+#include <errno.h>
 #include <string.h>
+
 #include <glib.h>
 #include <glib/gi18n-lib.h>
-
-#include "gucharmap-chapters.h"
+#include <gucharmap/gucharmap.h>
 #include "gucharmap-settings.h"
-#include "gucharmap-intl.h"
+
 #if HAVE_GCONF
 #include <gconf/gconf-client.h>
 static GConfClient *client;
 #endif
 
+/* The last unicode character we support */
+/* keep in sync with gucharmap-private.h! */
+#define UNICHAR_MAX (0x0010FFFFUL)
+
 #define WINDOW_STATE_TIMEOUT 1000 /* ms */
 
 #define GCONF_PREFIX "/apps/gucharmap"
 
-static gunichar
-get_first_non_underscore_char (const char *str)
-{
-  const char *p;
-
-  if (!str)
-    return 0;
-
-  for (p = str; p && *p; p = g_utf8_find_next_char (p, NULL))
-    {
-      gunichar ch;
-
-      ch = g_utf8_get_char (p);
-      if (g_unichar_isalpha (ch))
-        return ch;
-    }
-
-  return 0;
-}
-
-static gunichar
-get_default_last_char (void)
-{
-  return get_first_non_underscore_char (_("_File")); /* use a super-common string */
-}
-
-static ChaptersMode
+static GucharmapChaptersMode
 get_default_chapters_mode (void)
 {
   /* XXX: In the future, do something based on chapters mode and locale 
    * or something. */
-  return CHAPTERS_SCRIPT;
+  return GUCHARMAP_CHAPTERS_SCRIPT;
 }
 
 static gchar *
@@ -111,10 +90,10 @@ gucharmap_settings_initialized (void)
   return (client != NULL);
 }
 
-ChaptersMode
+GucharmapChaptersMode
 gucharmap_settings_get_chapters_mode (void)
 {
-  ChaptersMode ret;
+  GucharmapChaptersMode ret;
   gchar *mode;
   
   mode = gconf_client_get_string (client, GCONF_PREFIX"/chapters_mode", NULL);
@@ -122,9 +101,9 @@ gucharmap_settings_get_chapters_mode (void)
     return get_default_chapters_mode ();
 
   if (strcmp (mode, "script") == 0)
-    ret = CHAPTERS_SCRIPT;
+    ret = GUCHARMAP_CHAPTERS_SCRIPT;
   else if (strcmp (mode, "block") == 0)
-    ret = CHAPTERS_BLOCK;
+    ret = GUCHARMAP_CHAPTERS_BLOCK;
   else
     ret = get_default_chapters_mode ();
 
@@ -133,15 +112,15 @@ gucharmap_settings_get_chapters_mode (void)
 }
 
 void
-gucharmap_settings_set_chapters_mode (ChaptersMode mode)
+gucharmap_settings_set_chapters_mode (GucharmapChaptersMode mode)
 {
   switch (mode)
     {
-      case CHAPTERS_SCRIPT:
+      case GUCHARMAP_CHAPTERS_SCRIPT:
         gconf_client_set_string (client, GCONF_PREFIX"/chapters_mode", "script", NULL);
       break;
 
-      case CHAPTERS_BLOCK:
+      case GUCHARMAP_CHAPTERS_BLOCK:
         gconf_client_set_string (client, GCONF_PREFIX"/chapters_mode", "block", NULL);
       break;
     }
@@ -150,11 +129,19 @@ gucharmap_settings_set_chapters_mode (ChaptersMode mode)
 gchar *
 gucharmap_settings_get_font (void)
 {
+  char *font;
+
   if (!gucharmap_settings_initialized ()) {
       return get_default_font ();
   }
   
-  return gconf_client_get_string (client, GCONF_PREFIX"/font", NULL);
+  font = gconf_client_get_string (client, GCONF_PREFIX"/font", NULL);
+  if (!font || !font[0]) {
+    g_free (font);
+    return NULL;
+  }
+
+  return font;
 }
 
 void
@@ -171,25 +158,30 @@ gunichar
 gucharmap_settings_get_last_char (void)
 {
   /* See bug 469053 */
-  gchar *str;
-  gunichar c;
+  gchar *str, *endptr;
+  guint64 value;
 
   if (!gucharmap_settings_initialized ()) {
-      return get_default_last_char ();
+      return gucharmap_unicode_get_locale_character ();
   }
 
   str = gconf_client_get_string (client, GCONF_PREFIX"/last_char", NULL);
-  if (!str) {
-    return get_default_last_char ();
+  if (!str || !g_str_has_prefix (str, "U+")) {
+    g_free (str);
+    return gucharmap_unicode_get_locale_character  ();
   }
 
-  /* FIXME: use g_ascii_strtoull */
-  sscanf (str, "U+%X", &c);
-  g_free(str);
-  if (c > 0 && c < UNICHAR_MAX)
-    return c;
+  endptr = NULL;
+  errno = 0;
+  value = g_ascii_strtoull (str + 2 /* skip the "U+" */, &endptr, 16);
+  if (errno || endptr == str || value > UNICHAR_MAX) {
+    g_free (str);
+    return gucharmap_unicode_get_locale_character  ();
+  }
 
-  return get_default_last_char ();
+  g_free (str);
+
+  return (gunichar) value;
 }
 
 void
@@ -259,7 +251,7 @@ gucharmap_settings_get_chapters_mode (void)
 }
 
 void
-gucharmap_settings_set_chapters_mode (ChaptersMode mode)
+gucharmap_settings_set_chapters_mode (GucharmapChaptersMode mode)
 {
   return;
 }
@@ -279,7 +271,7 @@ gucharmap_settings_set_font (gchar *fontname)
 gunichar
 gucharmap_settings_get_last_char (void)
 {
-  return get_default_last_char ();
+  return gucharmap_unicode_get_locale_character  ();
 }
 
 void
